import React, { createContext, useContext } from 'react';
import { useProposalDepositForm } from '../forms/useProposalDepositForm';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';

export type ProposalDepositContextType = ReturnType<typeof useProposalDepositForm>;

export const ProposalDepositContext = createContext<ProposalDepositContextType>(null as any);

export function useProposalDeposit() {
  const state = useContext(ProposalDepositContext);
  if (!state) throw new Error('ProposalDepositContext missing');
  return state;
}

function defaultRenderLoading() {
  return <PlaceholderLoadingIndicator text="Loading proposal" />;
}

function defaultRenderError(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function ProposalDepositProvider({
  children,
  renderLoading = defaultRenderLoading,
  renderError = defaultRenderError,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof defaultRenderLoading;
  renderError?: typeof defaultRenderError;
} & Parameters<typeof useProposalDepositForm>[0]) {
  const state = useProposalDepositForm(props);
  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  if (!state.proposal) return renderLoading();
  return (
    <ProposalDepositContext.Provider value={state}>{children}</ProposalDepositContext.Provider>
  );
}
